package com.huiju.welcome.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.huiju.welcome.common.Action;
import com.huiju.welcome.common.WebSocketServer;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.*;
import com.huiju.welcome.service.*;
import com.huiju.welcome.utils.AliFaceUtils;
import com.huiju.welcome.utils.AliOSSUtils;
import com.huiju.welcome.utils.StatusUtils;
import com.huiju.welcome.utils.XFYun;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.annotation.PostConstruct;
import javax.servlet.http.HttpServletRequest;
import javax.websocket.server.PathParam;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author FXF
 * @date 2019-03-13
 */
@RestController
@RequestMapping("/")
@Transactional(rollbackFor = Exception.class)
public class CameraController {
	private Logger log = LoggerFactory.getLogger(XFYun.class);

	@Autowired
	ITaDeviceService taDeviceService;
	
	@Autowired
	private ISysParamService iSysParamService;

	@Autowired
	private ITaCarRecordService iTaCarRecordService;

	@Autowired
	private ICameraService cameraService;

	@Autowired
	private AliFaceUtils aliFaceUtils;

	@GetMapping("/ping")
	public String ping() {
		return "pong";
	}

	@ApiOperation(value = "人脸对比结果上传", notes = "人脸对比结果上传")
	@PostMapping(value = "/camera/{deviceId}/person")
	public ResponseBean person(
			@PathVariable String deviceId,
			MultipartFile snapshot,
			TaSnapshotPerson taSnapshotPerson,
			HttpServletRequest request) {
		ResponseBean responseBean = new ResponseBean();

		if (null == deviceId || null == taSnapshotPerson) {
			responseBean.addError(HttpStatus.SC_BAD_REQUEST, "设备ID 或者 抓拍参数传入方式不正确");
			return responseBean;
		}

		if (!deviceId.equals(taSnapshotPerson.getDeviceId())) {
			responseBean.addError(HttpStatus.SC_BAD_REQUEST, "设备ID 或者 抓拍参数传入不正确");
			return responseBean;
		}

		taSnapshotPerson.setAvatar(getMultipartFile(snapshot));

		// 抓拍时间为当前时间
		taSnapshotPerson.setCreateDate(LocalDateTime.now());

		cameraService.newPersonByDevice(taSnapshotPerson);
		return responseBean;
	}

	@PostMapping("/camera/{deviceId}/face")
	public ResponseBean face(
			@PathVariable("deviceId") String deviceId,
			@RequestParam(value = "avatar", required = false) MultipartFile avatar,
			@RequestParam(value = "score", defaultValue = "0") Integer score
	) {
		if (avatar.isEmpty()) {
			return ResponseBean.error("没有找到人脸信息", HttpStatus.SC_BAD_REQUEST);
		}

		// 上传阿里云OSS
		String avatarURL = getMultipartFile(avatar);

		// 人像质量最低值, 低于这个值的认为人脸抓拍失败, 取值范围 1 - 100
		Float faceMinScore = iSysParamService.getFaceMinScore();
		if (score < faceMinScore) {
			log.error("人脸质量过低, 放弃本次抓拍人脸 {}", avatarURL);
			return ResponseBean.success("");
		}

		TaSnapshotPerson taSnapshotPerson = new TaSnapshotPerson();

		// 人脸比对检索
		JSONObject result = null;
		try {
			result = aliFaceUtils.searchFace(avatarURL);
		} catch (Exception e) {
			log.error("比对人脸失败: {}", e.getMessage());
			e.printStackTrace();
			return ResponseBean.error("比对人脸失败", HttpStatus.SC_INTERNAL_SERVER_ERROR);
		}

		if (null != result) {
			Float rate = result.getFloat("rate");
			if (null == rate) {
				log.error("比对人脸异常, 比对阈值结果为空");
				rate = 0.0f;
			} else {
				rate *= 100;
			}
			Integer matchPerson = result.getInteger("personId");

			taSnapshotPerson.setSimilarity(rate);
			taSnapshotPerson.setMatchPerson(matchPerson);
		} else {
			taSnapshotPerson.setSimilarity(0f);
		}

		taSnapshotPerson.setCreateDate(LocalDateTime.now());
		taSnapshotPerson.setStatus(StatusUtils.Normal);
		taSnapshotPerson.setAvatar(avatarURL);
		taSnapshotPerson.setDeviceId(deviceId);
		taSnapshotPerson.setScore(score);

		cameraService.newPersonByDevice(taSnapshotPerson);
		return ResponseBean.success("");
	}

	@PostMapping(value = "/camera/{deviceId}/car")
	public ResponseBean car(
			@PathVariable("deviceId") String deviceId,
			@RequestParam("sceneImage") MultipartFile carImage,
			@RequestParam("licenseImage") MultipartFile licenseImage,
			@RequestParam(value = "driverImage", required = false) MultipartFile driverImage,
			@RequestParam(value = "copilotImage", required = false) MultipartFile copilotImage,
			@RequestParam("plateNumber") String plateNumber,
			@RequestParam("color") String color,
			@RequestParam("carModel") String carModel
			) {

		TaCarRecord taCarRecord = new TaCarRecord();
		taCarRecord.setPlateNumber(plateNumber);
		taCarRecord.setColor(color);
		taCarRecord.setCarModel(carModel);
		taCarRecord.setCreateDate(LocalDateTime.now());

		if (null != carImage && !carImage.isEmpty()) {
			taCarRecord.setCarImage(getMultipartFile(carImage));
		}
		if (null != licenseImage && !licenseImage.isEmpty()) {
			taCarRecord.setLicenseImage(getMultipartFile(licenseImage));
		}
		if (null != driverImage && !driverImage.isEmpty()) {
			taCarRecord.setDriverImage(getMultipartFile(driverImage));
		}
		if (null != copilotImage && !copilotImage.isEmpty()) {
			taCarRecord.setCopilotImage(getMultipartFile(copilotImage));
		}

		iTaCarRecordService.newRecordByDevice(taCarRecord);

		return ResponseBean.success("");
	}

	// 获取上传单文件
	private String getMultipartFile(MultipartFile f) {
		try {
			return AliOSSUtils.putObject(f);
		} catch (IOException e) {
			e.printStackTrace();
			log.error("上传人脸至阿里云失败: " + e.getMessage());
			return null;
		}
	}
}
