package com.huiju.welcome.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.SysUser;
import com.huiju.welcome.model.TaCustomer;
import com.huiju.welcome.model.TaFollowupRecord;
import com.huiju.welcome.model.TaMainUsherRecord;
import com.huiju.welcome.service.*;
import com.huiju.welcome.utils.DateUtils;
import com.huiju.welcome.utils.StatusUtils;
import com.huiju.welcome.utils.StringUtils;
import org.apache.http.HttpStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
    *  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2019-06-04
 */
@RestController
@RequestMapping("/")
public class TaMainUsherRecordController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaMainUsherRecordController.class);

    @Autowired
    private ITaMainUsherRecordService iTaMainUsherRecordService;

    @Autowired
    private ITaFollowupRecordService iTaFollowupRecordService;

    @Autowired
    private ITaPersonService iTaPersonService;

    /**
     * 保存对象
     * @param taMainUsherRecord 实体对象
     * @return
     */
    @RequestMapping(value="/miniapp/mainUsherRecord",method= RequestMethod.POST)
    public ResponseBean save(@RequestBody TaMainUsherRecord taMainUsherRecord){
        ResponseBean responseBean = new ResponseBean();
        try {
            iTaMainUsherRecordService.newRecordByMiniapp(taMainUsherRecord);

            responseBean.addSuccess(taMainUsherRecord);
        }catch (Exception e){
            logger.error("taMainUsherRecordAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/miniapp/mainUsherRecord/{id}/delete", method= RequestMethod.PUT)
    public ResponseBean delete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            iTaMainUsherRecordService.deleteRecord(id);
        }catch (Exception e){
            logger.error("taMainUsherRecordDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param taMainUsherRecord 实体对象
     * @return
     */
    @RequestMapping(value="/miniapp/mainUsherRecord/{id}",method= RequestMethod.PUT)
    public ResponseBean update(@RequestBody TaMainUsherRecord taMainUsherRecord, @PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            iTaMainUsherRecordService.updateByMiniapp(taMainUsherRecord);
        }catch (Exception e){
            logger.error("taMainUsherRecordUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/mainUsherRecord/{id}",method= RequestMethod.GET)
    public ResponseBean get(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            TaMainUsherRecord taMainUsherRecord = iTaMainUsherRecordService.getById(id);

            QueryWrapper<TaFollowupRecord> wrapper = new QueryWrapper<>();
            wrapper.eq("rec_id", id);

            List<TaFollowupRecord> list = iTaFollowupRecordService.list(wrapper);
            taMainUsherRecord.setFollows(list);

            responseBean.addSuccess(iTaMainUsherRecordService.getById(id));
        }catch (Exception e){
            logger.error("taMainUsherRecordGetByID -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    @PutMapping("/miniapp/mainUsherRecord/{id}/whitelist")
    public ResponseBean whitelist(@PathVariable Integer id, @RequestBody String paramStr) {
        JSONObject params = JSONObject.parseObject(paramStr);
        if (null == params) {
            return ResponseBean.error("白名单信息为空或者非法", HttpStatus.SC_BAD_REQUEST);
        }

        Integer personId = params.getInteger("personId");
        if (null == personId || personId.intValue() == 0) {
            return ResponseBean.error("未找到需要设置白对单对象", HttpStatus.SC_BAD_REQUEST);
        }

        iTaPersonService.setPersonIntoWhitelist(personId);
        iTaMainUsherRecordService.deleteRecord(id);

        return ResponseBean.success("");
    }

    /**
     * 分页查询
     */
    @RequestMapping(value="/miniapp/mainUsherRecord",method= RequestMethod.GET)
    public ResponseBean list(
        @RequestParam(defaultValue = "1") int pageNumber,
        @RequestParam(defaultValue = "10") int pageSize,
        @RequestParam(defaultValue = "false") boolean today,
        @RequestParam(defaultValue = "false") boolean done,
        @RequestParam(defaultValue = "false") boolean onlyNew,
        @RequestParam(defaultValue = "0") int fromRec,
        @RequestParam(defaultValue = "") String customer
    ){
        ResponseBean responseBean = new ResponseBean();
        IPage<TaMainUsherRecord> pg = new Page<>(pageNumber, pageSize);
        QueryWrapper<TaMainUsherRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.ge(today, "visite_date", DateUtils.morning());
        queryWrapper.gt("rec_id", fromRec);
        queryWrapper.eq(null != customer && !"".equals(customer), "customer_id", customer);

        if (onlyNew) {
            queryWrapper.eq("status", StatusUtils.Ready);
            queryWrapper.orderByAsc("rec_id");
        } else {
            queryWrapper.ge("status", StatusUtils.Ready);
            queryWrapper.lt(!done,"status", StatusUtils.BizDone);
            queryWrapper.orderByDesc("rec_id");
        }

        IPage<TaMainUsherRecord> result = iTaMainUsherRecordService.page(pg, queryWrapper);
        List<TaMainUsherRecord> recs = result.getRecords();

        if (null != recs && recs.size() > 0) {
            for (TaMainUsherRecord rec: result.getRecords()) {
                QueryWrapper<TaFollowupRecord> wrapper = new QueryWrapper<>();
                wrapper.eq("rec_id", rec.getRecId());
                rec.setFollows(iTaFollowupRecordService.list(wrapper));

                // 上次来访
                QueryWrapper<TaMainUsherRecord> query = new QueryWrapper<>();
                query.lt("visite_date", rec.getVisiteDate());
                query.lt("customer_id", rec.getCustomerId());
                query.orderByDesc("visite_date");
                query.last("limit 1");
                rec.setLastRecord(iTaMainUsherRecordService.getOne(query));
            }

//                if (onlyNew) {
//                    iTaMainUsherRecordService.updateNormal(recs);
//                }
        }
        responseBean.addSuccess(result);
        return responseBean;
    }

    @RequestMapping(value="/mainUsherRecord",method= RequestMethod.GET)
    public ResponseBean list(@RequestParam(defaultValue = "1") int pageNum,
                            @RequestParam(defaultValue = "10") int pageSize,
                             @RequestParam(value = "phone",required = false) String phone,
                             @RequestParam(value = "platNumber",required = false) String platNumber,
                             @RequestParam(value = "personName",required = false) String personName,
                             @RequestParam(value = "appointment",required = false) Integer appointment){
        ResponseBean responseBean = new ResponseBean();

        IPage<TaMainUsherRecord> pg = new Page<>(pageNum, pageSize);
        IPage<TaMainUsherRecord> result = iTaMainUsherRecordService.taMainUsherRecordList(pg,phone,platNumber,personName,appointment);
        responseBean.addSuccess(result);
        return responseBean;
    }

    @RequestMapping(value="/customerList",method= RequestMethod.GET)
    public ResponseBean customer(@RequestParam(defaultValue = "1") int pageNum,
                                 @RequestParam(defaultValue = "10") int pageSize,
                                 @RequestParam(value = "customerName" ,required = false)String customerName,
                                 @RequestParam(value = "phone" ,required = false)String phone,
                                 @RequestParam(value = "platNumber" ,required = false)String platNumber){
        ResponseBean result = iTaMainUsherRecordService.customerPage(pageNum, pageSize,customerName,phone,platNumber);
        return result;
    }
}
