package com.huiju.welcome.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.TaPersonType;
import com.huiju.welcome.service.ITaPersonTypeService;
import com.huiju.welcome.utils.StatusUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author jobob
 * @since 2019-03-12
 */
@RestController
@RequestMapping("/")
public class TaPersonTypeController extends BaseController {
    @Autowired
    private ITaPersonTypeService taPersonTypeService;

    @ApiOperation(value = "新增人员类型", notes = "新增人员类型")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "typeName", value = "人员类型名称")
    })
    @RequestMapping(value = "/person_type", method = RequestMethod.POST)
    public ResponseBean create(@RequestBody TaPersonType taPersonType, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        taPersonType.setStatus(StatusUtils.Normal);
        boolean success = taPersonTypeService.save(taPersonType);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "添加内容失败");
            return responseBean;
        }

        responseBean.addSuccess(taPersonType);
        return responseBean;
    }

    @ApiOperation(value = "更新人员类型", notes = "更新人员类型")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "typeName", value = "人员类型名称")
    })
    @RequestMapping(value = "/person_type", method = RequestMethod.PUT)
    public ResponseBean update(@RequestBody TaPersonType taPersonType, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        taPersonType.setStatus(StatusUtils.Normal);
        boolean success = taPersonTypeService.updateById(taPersonType);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "更新内容失败");
            return responseBean;
        }

        return responseBean;
    }

    @ApiOperation(value = "更新人员类型", notes = "更新人员类型")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", dataType = "Integer", name = "id", value = "人员类型ID")
    })
    @RequestMapping(value = "/person_type/{id}", method = RequestMethod.DELETE)
    public ResponseBean delete(@PathVariable int id, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        TaPersonType taPersonType = taPersonTypeService.getById(id);
        if (null == taPersonType) {
            responseBean.addError(HttpStatus.SC_NOT_FOUND, "未找到人员类型");
            return responseBean;
        }

        taPersonType.setStatus(StatusUtils.Delete);
        boolean success = taPersonTypeService.updateById(taPersonType);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "删除内容失败");
            return responseBean;
        }

        return responseBean;
    }

    @ApiOperation(value = "获取人员分类列表", notes = "获取人员分类列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "pageNum", value = "当前页"),
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "pageSize", value = "每页数量"),
    })
    @RequestMapping(value = "/person_type", method = RequestMethod.GET)
    public ResponseBean list (@RequestParam Integer pageNum, @RequestParam  Integer pageSize, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        Page<TaPersonType> pg = new Page<>();
        pg.setCurrent(pageNum);
        pg.setSize(pageSize);
        QueryWrapper<TaPersonType> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("`status`", StatusUtils.Normal);
        queryWrapper.orderByDesc("type_id");
        responseBean.addSuccess(taPersonTypeService.page(pg, queryWrapper));

        return responseBean;
    }

    @ApiOperation(value = "获取人员类型详情", notes = "获取人员类型详情")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", dataType = "Integer", name = "id", value = "人员类型ID")
    })
    @RequestMapping(value = "/person_type/{id}", method = RequestMethod.GET)
    public ResponseBean get(@PathVariable int id, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        TaPersonType taPersonType = taPersonTypeService.getById(id);
        if (null == taPersonType) {
            responseBean.addError(HttpStatus.SC_NOT_FOUND, "未找到人员类型");
            return responseBean;
        }

        responseBean.addSuccess(taPersonType);
        return responseBean;
    }
}
